@@ -65,6 +65,7 @@ typedef struct {
 	int chart_group;
 	gboolean  has_legend;
 	GogStyle *style;
+	GHashTable *singletons;
 } XLChartSeries;
 
 typedef struct {
@@ -117,6 +118,7 @@ excel_chart_series_new (void)
 	series->chart_group = -1;
 	series->has_legend = TRUE;
 	series->style = NULL;
+	series->singletons = NULL;
 	for (i = GOG_MS_DIM_TYPES; i-- > 0 ; ) {
 		series->data [i].data = NULL;
 		series->data [i].num_elements = 0;
@@ -135,6 +137,8 @@ excel_chart_series_delete (XLChartSeries *series)
 			g_object_unref (series->data[i].data);
 	if (series->style != NULL)
 		g_object_unref (series->style);
+	if (series->singletons != NULL)
+		g_hash_table_destroy (series->singletons);
 	g_free (series);
 }
 
@@ -768,10 +772,10 @@ BC_R(dataformat)(XLChartHandler const *handle,
 		d (0, fprintf (stderr, "All points"););
 	} else {
 		s->style_element = pt_num;
-		d (0, fprintf (stderr, "Point-%hd", pt_num););
+		d (0, fprintf (stderr, "Point[%hu]", pt_num););
 	}
 
-	d (0, fprintf (stderr, ", series=%hd\n", series_index););
+	d (0, fprintf (stderr, ", series=%hu\n", series_index););
 
 	return FALSE;
 }
@@ -944,7 +948,7 @@ BC_R(gelframe) (XLChartHandler const *handle,
 	} else if (type == 6) { /* fill from center */
 		/* TODO */
 	} else if (type == 7) {
-		GOGradientDirection dir;
+		GOGradientDirection dir = GO_GRADIENT_S_TO_N;
 		guint32 angle = ms_obj_attr_get_uint (attrs, MS_OBJ_ATTR_FILL_ANGLE, 0);
 		gint32 focus = ms_obj_attr_get_int (attrs, MS_OBJ_ATTR_FILL_FOCUS, 0);
 
@@ -1122,6 +1126,14 @@ static gboolean
 BC_R(lineformat)(XLChartHandler const *handle,
 		 XLChartReadState *s, BiffQuery *q)
 {
+	static GOLineDashType const dash_map []= {
+		GO_LINE_SOLID,
+		GO_LINE_DASH,
+		GO_LINE_DOT,
+		GO_LINE_DASH_DOT,
+		GO_LINE_DASH_DOT_DOT,
+		GO_LINE_NONE
+	};
 	guint16 const flags = GSF_LE_GET_GUINT16 (q->data+8);
 
 	BC_R(get_style) (s);
@@ -1145,8 +1157,10 @@ BC_R(lineformat)(XLChartHandler const *handle,
 	d (0, fprintf (stderr, "Lines have a %s pattern.\n",
 		       ms_line_pattern [s->style->line.pattern ]););
 
-	if (s->style->line.pattern == 5) /* invisible */
-		s->style->line.width = -1;
+	if (s->style->line.pattern <= G_N_ELEMENTS (dash_map))
+		s->style->line.dash_type = dash_map [s->style->line.pattern];
+	else
+		s->style->line.dash_type = GO_LINE_SOLID;
 
 	return FALSE;
 }
@@ -1311,17 +1325,20 @@ static gboolean
 BC_R(pieformat)(XLChartHandler const *handle,
 		XLChartReadState *s, BiffQuery *q)
 {
-	guint16 const default_separation = GSF_LE_GET_GUINT16 (q->data); /* 0-100 */
+	unsigned separation = GSF_LE_GET_GUINT16 (q->data); /* 0-500 */
 
 	/* we only support the default right now.  Also, XL sets this for _all_ types
 	 * rather than just pies. */
-	if (s->plot != NULL &&
+	if (s->style_element >= 0 && s->style != NULL)
+		g_object_set_data (G_OBJECT (s->style),
+			"pie-separation", GUINT_TO_POINTER (separation));
+	else if (s->plot != NULL &&
 	    g_object_class_find_property (G_OBJECT_GET_CLASS (s->plot), "default_separation"))
 		g_object_set (G_OBJECT (s->plot),
-			"default_separation",	((double) default_separation) / 100.,
+			"default_separation",	((double) separation) / 100.,
 			NULL);
-	d (2, fprintf (stderr, "Pie slice(s) are %hu %% of diam from center\n",
-		       default_separation););
+	d (2, fprintf (stderr, "Pie slice(s) are %u %% of diam from center\n",
+		       separation););
 	return FALSE;
 }
 
@@ -1389,10 +1406,7 @@ BC_R(radararea)(XLChartHandler const *handle,
 		XLChartReadState *s, BiffQuery *q)
 {
 	g_return_val_if_fail (s->plot == NULL, TRUE);
-	s->plot = gog_plot_new_by_name ("GogRadarPlot");
-	g_object_set (G_OBJECT (s->plot),
-		"area",		TRUE,
-		NULL);
+	s->plot = gog_plot_new_by_name ("GogRadarAreaPlot");
 
 	return FALSE;
 }
@@ -1839,6 +1853,36 @@ XL_gog_series_set_dim (GogSeries *series, GogMSDimType ms_type, GOData *val)
 	g_object_unref (val);
 }
 
+static void
+cb_store_singletons (gpointer indx, GogStyle *style, GogObject *series)
+{
+	GogObject *singleton = gog_object_add_by_name (series, "Point", NULL);
+	if (singleton != NULL) {
+		g_object_set (singleton,
+			"index", GPOINTER_TO_UINT (indx),
+			"style", style,
+			NULL);
+		if (g_object_class_find_property (G_OBJECT_GET_CLASS (singleton), "separation")) {
+			gpointer sep = g_object_get_data (G_OBJECT (style), "pie-separation");
+			g_object_set (singleton,
+				"separation", (double)GPOINTER_TO_UINT (sep) / 100.,
+				NULL);
+		}
+	}
+}
+
+static void
+xl_axis_swap_elem (GogAxis *a, GogAxis *b, unsigned dim)
+{
+	GOData *a_dat = gog_dataset_get_dim (GOG_DATASET (a), dim);
+	GOData *b_dat = gog_dataset_get_dim (GOG_DATASET (b), dim);
+
+	if (a_dat != NULL) g_object_ref (a_dat);
+	if (b_dat != NULL) g_object_ref (b_dat);
+	gog_dataset_set_dim (GOG_DATASET (a), dim, b_dat, NULL);
+	gog_dataset_set_dim (GOG_DATASET (b), dim, a_dat, NULL);
+}
+
 static gboolean
 BC_R(end)(XLChartHandler const *handle,
 	  XLChartReadState *s, BiffQuery *q)
@@ -1941,16 +1985,31 @@ BC_R(end)(XLChartHandler const *handle,
 				g_object_set (G_OBJECT (series),
 					"has-legend", FALSE,
 					NULL);
+			if (eseries->singletons != NULL)
+				g_hash_table_foreach (eseries->singletons,
+					(GHFunc) cb_store_singletons, series);
 		}
 
+		/* Vile cheesy hack.
+		 * XL stores axis as 'value' and 'category' whereas we use X and Y.
+		 * When importing bar plots things are transposed, but we do
+		 * not know it until we import the plot.  Swap the contents of the axes */
 		if (0 == strcmp (G_OBJECT_TYPE_NAME (s->plot), "GogBarColPlot")) {
 			gboolean horizontal;
 			g_object_get (s->plot, "horizontal", &horizontal, NULL);
 			if (horizontal) {
-				g_warning ("the axes will be reversed.  fix this");
-#if 0
-				gog_chart_swap_xy_axes (s->chart);
-#endif
+				GogAxis	*x = gog_plot_get_axis (s->plot, GOG_AXIS_X);
+				GogAxis	*y = gog_plot_get_axis (s->plot, GOG_AXIS_Y);
+				GogStyle *x_style, *y_style;
+				int i;
+				for (i = 0 ; i < AXIS_ELEM_MAX_ENTRY ; i++)
+					xl_axis_swap_elem (x, y, i);
+				g_object_get (G_OBJECT (x), "style", &x_style, NULL);
+				g_object_get (G_OBJECT (y), "style", &y_style, NULL);
+				g_object_set (G_OBJECT (y), "style", x_style, NULL);
+				g_object_set (G_OBJECT (x), "style", y_style, NULL);
+				g_object_unref (x_style);
+				g_object_unref (y_style);
 			}
 		}
 		s->plot = NULL;
@@ -1960,9 +2019,17 @@ BC_R(end)(XLChartHandler const *handle,
 	case BIFF_CHART_dataformat :
 		if (s->style == NULL)
 			break;
-		if (s->currentSeries != NULL && s->style_element < 0) {
-			g_return_val_if_fail (s->currentSeries->style == NULL, TRUE);
-			s->currentSeries->style = s->style;
+		if (s->currentSeries != NULL) {
+			if (s->style_element < 0) {
+				g_return_val_if_fail (s->currentSeries->style == NULL, TRUE);
+				s->currentSeries->style = s->style;
+			} else {
+				if (s->currentSeries->singletons == NULL)
+					s->currentSeries->singletons = g_hash_table_new_full (
+						g_direct_hash, g_direct_equal, NULL, g_object_unref);
+				g_hash_table_insert (s->currentSeries->singletons,
+					GUINT_TO_POINTER (s->style_element), s->style);
+			}
 		} else if (s->plot != NULL) {
 			g_return_val_if_fail (s->default_plot_style == NULL, TRUE);
 			s->default_plot_style = s->style;
@@ -2172,7 +2239,8 @@ ms_excel_chart_read (BiffQuery *q, MSContainer *container, MsBiffVersion ver,
 
 	if (NULL != (state.sog = sog)) {
 		GogStyle *style = gog_style_new ();
-		style->outline.width = -1;
+		style->outline.width = 0;
+		style->outline.dash_type = GO_LINE_NONE;
 		style->fill.type = GOG_FILL_STYLE_NONE;
 
 		state.graph = sheet_object_graph_get_gog (sog);
@@ -2193,10 +2261,9 @@ ms_excel_chart_read (BiffQuery *q, MSContainer *container, MsBiffVersion ver,
 	d (0, fputs ("{ /* CHART */\n", stderr););
 
 	while (!done && ms_biff_query_next (q)) {
-		int const lsb = q->opcode & 0xff;
-
 		/* Use registered jump table for chart records */
 		if ((q->opcode & 0xff00) == 0x1000) {
+			int const lsb = q->opcode & 0xff;
 			int const begin_end =
 				(q->opcode == BIFF_CHART_begin ||
 				 q->opcode == BIFF_CHART_end);
@@ -2218,87 +2285,91 @@ ms_excel_chart_read (BiffQuery *q, MSContainer *container, MsBiffVersion ver,
 							fprintf (stderr, ");\n"); });
 				}
 			}
-		} else {
-			switch (lsb) {
-			case BIFF_EOF:
-				done = TRUE;
-				d (0, fputs ("}; /* CHART */\n", stderr););
-				g_return_val_if_fail(state.stack->len == 0, TRUE);
-				break;
-
-			case BIFF_PROTECT : {
-				gboolean const is_protected =
-					(1 == GSF_LE_GET_GUINT16 (q->data));
-				d (4, fprintf (stderr, "Chart is%s protected;\n",
-					     is_protected ? "" : " not"););
-				break;
-			}
-
-			case BIFF_NUMBER: {
-				double val;
-				val = gsf_le_get_double (q->data + 6);
-				/* Figure out how to assign these back to the series,
-				 * are they just sequential ?
-				 */
-				d (10, fprintf (stderr, "%f\n", val););
-				break;
-			}
-
-			case BIFF_LABEL : {
-				guint16 row = GSF_LE_GET_GUINT16 (q->data + 0);
-				guint16 col = GSF_LE_GET_GUINT16 (q->data + 2);
-				guint16 xf  = GSF_LE_GET_GUINT16 (q->data + 4);
-				guint16 len = GSF_LE_GET_GUINT16 (q->data + 6);
-				char *label = biff_get_text (q->data + 8, len, NULL, ver);
-				d (10, {fprintf (stderr, "'%s'\n;hmm, what are these values for a chart ???\n"
-						"row = %d, col = %d, xf = %d\n", label, row, col, xf);});
-				g_free (label);
-				break;
-			}
-
-			case BIFF_MS_O_DRAWING:
-				ms_escher_parse (q, &state.container, FALSE);
-				break;
-
-			case BIFF_EXTERNCOUNT: /* ignore */ break;
-			case BIFF_EXTERNSHEET: /* These cannot be biff8 */
-				excel_read_EXTERNSHEET_v7 (q, &state.container);
-				break;
-
-			case BIFF_WINDOW2 :
-				if (full_page != NULL && container->ver > MS_BIFF_V2)
-					if (GSF_LE_GET_GUINT16 (q->data + 0) & 0x0400)
-						wb_view_sheet_focus (container->ewb->wbv, full_page);
-				break;
-
-			case BIFF_SCL :
-				if (full_page != NULL)
-					excel_read_SCL (q, full_page);
-				break;
-
-			case BIFF_PLS:		/* Skip for Now */
-			case BIFF_DIMENSIONS :	/* Skip for Now */
-			case BIFF_HEADER :	/* Skip for Now */
-			case BIFF_FOOTER :	/* Skip for Now */
-			case BIFF_HCENTER :	/* Skip for Now */
-			case BIFF_VCENTER :	/* Skip for Now */
-			case BIFF_CODENAME :
-			case BIFF_SETUP :
-				d (8, fprintf (stderr, "Handled biff %x in chart;\n",
-					     q->opcode););
-				break;
-
-			case BIFF_PRINTSIZE: {
-#if 0
-				/* Undocumented, seems like an enum ??? */
-				gint16 const v = GSF_LE_GET_GUINT16 (q->data);
-#endif
-			}
+		} else switch (q->opcode) {
+		case BIFF_EOF:
+			done = TRUE;
+			d (0, fputs ("}; /* CHART */\n", stderr););
+			g_return_val_if_fail(state.stack->len == 0, TRUE);
 			break;
 
-			default :
-				excel_unexpected_biff (q, "Chart", ms_excel_chart_debug);
-			}
+		case BIFF_PROTECT : {
+			gboolean const is_protected =
+				(1 == GSF_LE_GET_GUINT16 (q->data));
+			d (4, fprintf (stderr, "Chart is%s protected;\n",
+				     is_protected ? "" : " not"););
+			break;
+		}
+
+		case BIFF_BLANK_v0:
+		case BIFF_BLANK_v2: /* Stores a missing value in the inline value tables */
+			break;
+		case BIFF_NUMBER_v0:
+		case BIFF_NUMBER_v2: {
+			unsigned offset = (q->opcode == BIFF_NUMBER_v2) ? 6: 7;
+			double val = gsf_le_get_double (q->data + offset);
+			/* Figure out how to assign these back to the series,
+			 * are they just sequential ?  */
+			d (10, fprintf (stderr, "%f\n", val););
+			break;
+		}
+
+		case BIFF_LABEL_v0 : break; /* ignore for now */
+		case BIFF_LABEL_v2 : {
+			guint16 row = GSF_LE_GET_GUINT16 (q->data + 0);
+			guint16 col = GSF_LE_GET_GUINT16 (q->data + 2);
+			guint16 xf  = GSF_LE_GET_GUINT16 (q->data + 4);
+			guint16 len = GSF_LE_GET_GUINT16 (q->data + 6);
+			char *label = biff_get_text (q->data + 8, len, NULL, ver);
+			d (10, {fprintf (stderr, "'%s'\n;hmm, what are these values for a chart ???\n"
+					"row = %d, col = %d, xf = %d\n", label, row, col, xf);});
+			g_free (label);
+			break;
+		}
+
+		case BIFF_MS_O_DRAWING:
+			ms_escher_parse (q, &state.container, FALSE);
+			break;
+
+		case BIFF_EXTERNCOUNT: /* ignore */ break;
+		case BIFF_EXTERNSHEET: /* These cannot be biff8 */
+			excel_read_EXTERNSHEET_v7 (q, &state.container);
+			break;
+
+		case BIFF_WINDOW2_v0 :
+		case BIFF_WINDOW2_v2 :
+			if (full_page != NULL && container->ver > MS_BIFF_V2)
+				if (GSF_LE_GET_GUINT16 (q->data + 0) & 0x0400)
+					wb_view_sheet_focus (container->ewb->wbv, full_page);
+			break;
+
+		case BIFF_SCL :
+			if (full_page != NULL)
+				excel_read_SCL (q, full_page);
+			break;
+
+		case BIFF_PLS:		/* Skip for Now */
+		case BIFF_DIMENSIONS_v0 :/* Skip for Now */
+		case BIFF_DIMENSIONS_v2 :/* Skip for Now */
+		case BIFF_HEADER :	/* Skip for Now */
+		case BIFF_FOOTER :	/* Skip for Now */
+		case BIFF_HCENTER :	/* Skip for Now */
+		case BIFF_VCENTER :	/* Skip for Now */
+		case BIFF_CODENAME :
+		case BIFF_SETUP :
+			d (8, fprintf (stderr, "Handled biff %x in chart;\n",
+				     q->opcode););
+			break;
+
+		case BIFF_PRINTSIZE: {
+#if 0
+			/* Undocumented, seems like an enum ??? */
+			gint16 const v = GSF_LE_GET_GUINT16 (q->data);
+#endif
+		}
+		break;
+
+		default :
+			excel_unexpected_biff (q, "Chart", ms_excel_chart_debug);
 		}
 		state.prev_opcode = q->opcode;
 	}
@@ -2327,7 +2398,7 @@ ms_excel_chart_read (BiffQuery *q, MSContainer *container, MsBiffVersion ver,
 		SheetObjectAnchor anchor;
 		sheet_object_anchor_init (&anchor,
 			&fixed_size, NULL, NULL, SO_DIR_DOWN_RIGHT);
-		sheet_object_anchor_set (sog, &anchor);
+		sheet_object_set_anchor (sog, &anchor);
 		sheet_object_set_sheet (sog, full_page);
 		g_object_unref (sog);
 	}
@@ -2775,6 +2846,25 @@ style_is_completely_auto (GogStyle const *style)
 	return TRUE;
 }
 
+static void
+chart_write_style (XLChartWriteState *s, GogStyle const *style,
+		   guint16 indx, unsigned n, float separation)
+{
+	chart_write_DATAFORMAT (s, indx, n, n);
+	chart_write_BEGIN (s);
+	ms_biff_put_2byte (s->bp, BIFF_CHART_3dbarshape, 0); /* box */
+	if (!style_is_completely_auto (style)) {
+		if ((style->interesting_fields & GOG_STYLE_LINE))
+			chart_write_LINEFORMAT (s, &style->line, FALSE, FALSE);
+		else
+			chart_write_LINEFORMAT (s, &style->outline, FALSE, FALSE);
+		chart_write_AREAFORMAT (s, style, FALSE);
+		chart_write_PIEFORMAT (s, separation);
+		chart_write_MARKERFORMAT (s, style, FALSE);
+	}
+	chart_write_END (s);
+}
+
 static void
 chart_write_series (XLChartWriteState *s, GogSeries const *series, unsigned n)
 {
@@ -2782,9 +2872,9 @@ chart_write_series (XLChartWriteState *s, GogSeries const *series, unsigned n)
 	int i, msdim;
 	guint8 *data;
 	GOData *dat;
-	GogStyle const *style;
 	unsigned num_elements = gog_series_num_elements (series);
-	
+	GList const *ptr;
+
 	/* SERIES */
 	data = ms_biff_put_len_next (s->bp, BIFF_CHART_series,
 		(s->bp->version >= MS_BIFF_V8) ? 12: 8);
@@ -2808,20 +2898,16 @@ chart_write_series (XLChartWriteState *s, GogSeries const *series, unsigned n)
 		chart_write_AI (s, dat, i, default_ref_type[i]);
 	}
 
-	chart_write_DATAFORMAT (s, 0xffff, n, n); /* ignore single elements for now */
-	chart_write_BEGIN (s);
-	ms_biff_put_2byte (s->bp, BIFF_CHART_3dbarshape, 0); /* box */
-	style = GOG_STYLED_OBJECT (series)->style;
-	if (!style_is_completely_auto (style)) {
-		if ((style->interesting_fields & GOG_STYLE_LINE))
-			chart_write_LINEFORMAT (s, &style->line, FALSE, FALSE);
-		else
-			chart_write_LINEFORMAT (s, &style->outline, FALSE, FALSE);
-		chart_write_AREAFORMAT (s, style, FALSE);
-		chart_write_PIEFORMAT (s, 0.);
-		chart_write_MARKERFORMAT (s, style, FALSE);
+	chart_write_style (s, GOG_STYLED_OBJECT (series)->style, 0xffff, n, 0.);
+	for (ptr = gog_series_get_overrides (series); ptr != NULL ; ptr = ptr->next) {
+		float sep = 0;
+		if (g_object_class_find_property (
+			G_OBJECT_GET_CLASS (ptr->data), "separation"))
+			g_object_get (G_OBJECT (ptr->data), "separation", &sep, NULL);
+
+		chart_write_style (s, GOG_STYLED_OBJECT (ptr->data)->style,
+			GOG_SERIES_ELEMENT (ptr->data)->index, n, sep);
 	}
-	chart_write_END (s);
 
 	ms_biff_put_2byte (s->bp, BIFF_CHART_sertocrt, 0);
 	chart_write_END (s);
@@ -3074,12 +3160,9 @@ chart_write_plot (XLChartWriteState *s, GogPlot const *plot)
 		if (fabs (default_separation) > .005)
 			chart_write_dummy_style (s, default_separation, FALSE, FALSE);
 	} else if (0 == strcmp (type, "GogRadarPlot")) {
-		gboolean area;
-		g_object_get (G_OBJECT (plot), "area", &area, NULL);
-		/* TODO : flags : chart contains radar axis labels */
-		ms_biff_put_2byte (s->bp,
-			area ? BIFF_CHART_radararea : BIFF_CHART_radar,
-			flags);
+		ms_biff_put_2byte (s->bp, BIFF_CHART_radar, flags);
+	} else if (0 == strcmp (type, "GogRadarAreaPlot")) {
+		ms_biff_put_2byte (s->bp, BIFF_CHART_radararea, flags);
 	} else if (0 == strcmp (type, "GogBubblePlot") ||
 		   0 == strcmp (type, "GogXYPlot")) {
 		if (s->bp->version >= MS_BIFF_V8) {
@@ -3210,7 +3293,7 @@ chart_write_axis_sets (XLChartWriteState *s, GSList *sets)
 
 			/* BIFF_CHART_chartformatlink documented as unnecessary */
 			if (i == 0 && legend != NULL) {
-				GogObjectPosition pos = gog_object_get_pos (legend);
+				/*GogObjectPosition pos = gog_object_get_pos (legend); */
 				guint16 flags = 0x1f;
 
 				data = ms_biff_put_len_next (s->bp, BIFF_CHART_legend, 20);
@@ -3244,7 +3327,7 @@ ms_excel_chart_write (ExcelWriteState *ewb, SheetObject *so)
 	XLChartWriteState state;
 	unsigned i, num_series = 0;
 	GSList const *plots, *series;
-	GSList *sets, *ptr;
+	GSList *charts, *sets, *ptr;
 	XLAxisSet *axis_set;
 
 	state.bp  = ewb->bp;
@@ -3252,17 +3335,23 @@ ms_excel_chart_write (ExcelWriteState *ewb, SheetObject *so)
 	state.so  = so;
 	state.graph = sheet_object_graph_get_gog (so);
 
-	/* WARNING : catch multiple charts */
-	state.chart = gog_object_get_child_by_role (GOG_OBJECT (state.graph),
+	g_return_if_fail (state.graph != NULL);
+
+	charts = gog_object_get_children (GOG_OBJECT (state.graph),
 		gog_object_find_role_by_name (GOG_OBJECT (state.graph), "Chart"));
+
+	g_return_if_fail (charts != NULL);
+
+	/* TODO : handle multiple charts */
+	state.chart = charts->data;
 	state.nest_level = 0;
-#warning TODO : create a null renderer class for use in sizing things
+
+	/* TODO : create a null renderer class for use in sizing things */
 	renderer  = g_object_new (GOG_RENDERER_TYPE,
 				  "model", state.graph,
 				  NULL);
 	g_object_get (G_OBJECT (renderer), "view", &state.root_view, NULL);
 
-	g_return_if_fail (state.graph != NULL);
 
 	excel_write_BOF (state.bp, MS_BIFF_TYPE_Chart);
 	ms_biff_put_empty (state.bp, BIFF_HEADER);
