@@ -121,19 +121,19 @@ gog_2d_plot_update (GogObject *obj)
 	gog_2d_plot_adjust_bounds (model, &x_min, &x_max, &y_min, &y_max);
 	/* add room for error bars */
 	if (gog_error_bar_is_visible (series->x_errors)) {
-			gog_error_bar_get_minmax (series->x_errors, &tmp_min, &tmp_max);
-			if (x_min > tmp_min)
-				x_min = tmp_min;
-			if (x_max < tmp_max)
-				x_max = tmp_max;
-		}
+		gog_error_bar_get_minmax (series->x_errors, &tmp_min, &tmp_max);
+		if (x_min > tmp_min)
+			x_min = tmp_min;
+		if (x_max < tmp_max)
+			x_max = tmp_max;
+	}
 	if (gog_error_bar_is_visible (series->y_errors)) {
-			gog_error_bar_get_minmax (series->y_errors, &tmp_min, &tmp_max);
-			if (y_min > tmp_min)
-				y_min = tmp_min;
-			if (y_max < tmp_max)
-				y_max = tmp_max;
-		}
+		gog_error_bar_get_minmax (series->y_errors, &tmp_min, &tmp_max);
+		if (y_min > tmp_min)
+			y_min = tmp_min;
+		if (y_max < tmp_max)
+			y_max = tmp_max;
+	}
 	
 	if (model->x.minima != x_min || model->x.maxima != x_max) {
 		model->x.minima = x_min;
@@ -379,11 +379,10 @@ gog_bubble_plot_type_name (G_GNUC_UNUSED GogObject const *item)
 	return N_("PlotBubble");
 }
 
-extern gpointer gog_bubble_plot_pref (GogBubblePlot *pie, GnmCmdContext *cc);
+extern gpointer gog_bubble_plot_pref (GogBubblePlot *bubble, GnmCmdContext *cc);
 static gpointer
-gog_bubble_plot_editor (GogObject *item,
-		    G_GNUC_UNUSED GogDataAllocator *dalloc,
-		    GnmCmdContext *cc)
+gog_bubble_plot_editor (GogObject *item, G_GNUC_UNUSED GogDataAllocator *dalloc,
+			GnmCmdContext *cc)
 {
 	return gog_bubble_plot_pref (GOG_BUBBLE_PLOT (item), cc);
 }
@@ -939,9 +938,11 @@ gog_xy_series_init_style (GogStyledObject *gso, GogStyle *style)
 	}
 	if (!plot->default_style_has_lines) {
 		style->disable_theming |= GOG_STYLE_LINE;
+		if (style->line.auto_dash)
+			style->line.dash_type = GO_LINE_NONE;
 		if (style->line.auto_color) {
-			style->line.width = -1;
-			style->line.color = 0;
+			style->line.width = 0;
+			style->line.color = RGBA_BLACK;
 		}
 	}
 }
