@@ -109,7 +109,7 @@ typedef enum {
 	GNM_FUNC_IMPL_STATUS_SUPERSET,
 	GNM_FUNC_IMPL_STATUS_SUBSET_WITH_EXTENSIONS,
 	GNM_FUNC_IMPL_STATUS_UNDER_DEVELOPMENT,
-	GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC,
+	GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC
 } GnmFuncImplStatus;
 
 typedef enum {
