@@ -11,7 +11,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "plugin.h"
 
@@ -32,12 +32,9 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/wait.h>
 #include <sys/time.h>
-#include <sys/utsname.h>
 #include <unistd.h>
 #include <dirent.h>
-#include <fnmatch.h>
 #include <locale.h>
 #include <gmodule.h>
 #include <application.h>
@@ -144,7 +141,7 @@ gnm_plugin_finalize (GObject *obj)
 		plugin->has_full_info = FALSE;
 		g_free (plugin->name);
 		g_free (plugin->description);
-		g_slist_free_custom (plugin->dependencies, plugin_dependency_free);
+		gnm_slist_free_custom (plugin->dependencies, plugin_dependency_free);
 		g_free (plugin->loader_id);
 		if (plugin->loader_attrs != NULL) {
 			g_hash_table_destroy (plugin->loader_attrs);
@@ -152,7 +149,7 @@ gnm_plugin_finalize (GObject *obj)
 		if (plugin->loader != NULL) {
 			g_object_unref (plugin->loader);
 		}
-		g_slist_free_custom (plugin->services, g_object_unref);
+		gnm_slist_free_custom (plugin->services, g_object_unref);
 	}
 	g_free (plugin->saved_textdomain);
 	plugin->saved_textdomain = NULL;
@@ -657,7 +654,7 @@ plugin_info_read_service_list (GnmPlugin *plugin, xmlNode *tree, ErrorInfo **ret
 	if (error_list != NULL) {
 		GNM_SLIST_REVERSE (error_list);
 		*ret_error = error_info_new_from_error_list (error_list);
-		g_slist_free_custom (service_list, g_object_unref);
+		gnm_slist_free_custom (service_list, g_object_unref);
 		return NULL;
 	} else {
 		return g_slist_reverse (service_list);
@@ -744,8 +741,7 @@ plugin_info_read (GnmPlugin *plugin, const gchar *dir_name, ErrorInfo **ret_erro
 		xmlNode *node;
 		xmlChar *val;
 
-		node = e_xml_get_child_by_name_by_lang_list (
-		       information_node, "name", NULL);
+		node = e_xml_get_child_by_name_by_lang (information_node, "name");
 		if (node != NULL) {
 			val = xmlNodeGetContent (node);
 			name = g_strdup ((gchar *)val);
@@ -753,8 +749,7 @@ plugin_info_read (GnmPlugin *plugin, const gchar *dir_name, ErrorInfo **ret_erro
 		} else
 			name = NULL;
 
-		node = e_xml_get_child_by_name_by_lang_list (
-		       information_node, "description", NULL);
+		node = e_xml_get_child_by_name_by_lang (information_node, "description");
 		if (node != NULL) {
 			val = xmlNodeGetContent (node);
 			description = g_strdup ((gchar *)val);
@@ -849,7 +844,7 @@ plugin_info_read (GnmPlugin *plugin, const gchar *dir_name, ErrorInfo **ret_erro
 		} else
 			*ret_error = error_info_new_str (_("Plugin has no id."));
 
-		g_slist_free_custom (dependency_list, plugin_dependency_free);
+		gnm_slist_free_custom (dependency_list, plugin_dependency_free);
 		g_free (plugin->loader_id);
 		if (plugin->loader_attrs != NULL)
 			g_hash_table_destroy (plugin->loader_attrs);
@@ -1232,7 +1227,7 @@ gnm_plugin_use_unref (GnmPlugin *plugin)
  * All these plugins will be automatically activated before activating
  * the @plugin itself.
  * The caller must free the returned list together with the strings it
- * points to (use g_slist_free_custom (list, g_free) to do this).
+ * points to (use gnm_slist_free_custom (list, g_free) to do this).
  */
 GSList *
 gnm_plugin_get_dependencies_ids (GnmPlugin *plugin)
@@ -1401,10 +1396,10 @@ gnumeric_extra_plugin_dirs (void)
 	GSList *extra_dirs;
 	gchar const *plugin_path_env;
 
-	extra_dirs = g_string_slist_copy (gnm_app_prefs->plugin_extra_dirs);
+	extra_dirs = gnm_string_slist_copy (gnm_app_prefs->plugin_extra_dirs);
 	plugin_path_env = g_getenv ("GNUMERIC_PLUGIN_PATH");
 	if (plugin_path_env != NULL) {
-		GNM_SLIST_CONCAT (extra_dirs, g_strsplit_to_slist (plugin_path_env, ":"));
+		GNM_SLIST_CONCAT (extra_dirs, gnm_strsplit_to_slist (plugin_path_env, ":"));
 	}
 
 	return extra_dirs;
@@ -1421,9 +1416,9 @@ plugin_info_list_read_for_all_dirs (ErrorInfo **ret_error)
 	ErrorInfo *error;
 
 	GNM_INIT_RET_ERROR_INFO (ret_error);
-	dir_list = g_create_slist (gnumeric_sys_plugin_dir (),
-				   gnumeric_usr_plugin_dir (),
-				   NULL);
+	dir_list = gnm_slist_create (gnm_sys_plugin_dir (),
+				     gnm_usr_plugin_dir (),
+				     NULL);
 	GNM_SLIST_CONCAT (dir_list, gnumeric_extra_plugin_dirs ());
 	plugin_info_list = plugin_info_list_read_for_subdirs_of_dir_list (dir_list, &error);
 	g_slist_foreach (dir_list, (GFunc)g_free, NULL);
@@ -1643,7 +1638,7 @@ plugins_rescan (ErrorInfo **ret_error, GSList **ret_new_plugins)
 			_("The following plugins are no longer on disk but are still active:\n"
 			  "%s.\nYou should restart Gnumeric now."), s->str));
 		g_string_free (s, TRUE);
-		g_slist_free_custom (still_active_ids, g_free);
+		gnm_slist_free_custom (still_active_ids, g_free);
 	}
 
 	/* Find previously not available plugins */
@@ -1657,7 +1652,7 @@ plugins_rescan (ErrorInfo **ret_error, GSList **ret_new_plugins)
 			g_object_ref (plugin);
 		}
 	);
-	g_slist_free_custom (new_available_plugins, g_object_unref);
+	gnm_slist_free_custom (new_available_plugins, g_object_unref);
 	if (ret_new_plugins != NULL) {
 		*ret_new_plugins = g_slist_copy (added_plugins);
 	}
@@ -1698,7 +1693,7 @@ plugins_init (GnmCmdContext *context)
 	ErrorInfo *error;
 	GSList *plugin_list;
 
-	gnumeric_time_counter_push ();
+	gnm_time_counter_push ();
 
 	loader_services = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
@@ -1760,7 +1755,7 @@ plugins_init (GnmCmdContext *context)
 		error_info_free (error);
 	}
 
-	plugin_message (4, "plugins_init() time: %fs\n", gnumeric_time_counter_pop ());
+	plugin_message (4, "plugins_init() time: %fs\n", gnm_time_counter_pop ());
 }
 
 static void
@@ -1835,14 +1830,14 @@ plugins_shutdown (void)
 		gnm_gconf_set_plugin_file_states (used_plugin_state_strings);
 		plugin_message (5, "Plugin cache changed\n");
 	} else
-		g_slist_free_custom (used_plugin_state_strings, g_free);
+		gnm_slist_free_custom (used_plugin_state_strings, g_free);
 
 	g_hash_table_destroy (plugin_file_state_dir_hash);
 	g_hash_table_destroy (loader_services);
 	g_hash_table_destroy (available_plugins_id_hash);
-	g_slist_free_custom (available_plugins, gnm_plugin_try_unref);
+	gnm_slist_free_custom (available_plugins, gnm_plugin_try_unref);
 
-	gnm_conf_sync ();
+	go_conf_sync ();
 }
 
 void
