@@ -24,17 +24,18 @@
 #include "ranges.h"
 #include "commands.h"
 
-#include "sheet-object-graphic.h"
+#include "gnm-so-line.h"
+#include "gnm-so-filled.h"
 #include "sheet-object-cell-comment.h"
 #include "sheet-object-widget.h"
 #include "sheet-object-graph.h"
 #include "sheet-object-image.h"
 #include "graph.h"
 #include "io-context.h"
+#include "application.h"
+#include "gutils.h"
 
 #include <libxml/globals.h>
-#include <gtk/gtkimagemenuitem.h>
-#include <gtk/gtkstock.h>
 #include <libfoocanvas/foo-canvas.h>
 #include <gsf/gsf-impl-utils.h>
 
@@ -43,149 +44,81 @@
 /* Returns the class for a SheetObject */
 #define SO_CLASS(so) SHEET_OBJECT_CLASS(G_OBJECT_GET_CLASS(so))
 
-#define	SO_VIEW_SHEET_CONTROL_KEY	"SheetControl"
-#define	SO_VIEW_OBJECT_KEY		"SheetObject"
-#define	SO_VIEW_KEY			"Key"
-
 enum {
 	BOUNDS_CHANGED,
+	UNREALIZED,
 	LAST_SIGNAL
 };
 static guint	     signals [LAST_SIGNAL] = { 0 };
 static GObjectClass *parent_klass;
+static GQuark	sov_so_quark;
+static GQuark	sov_container_quark;
 
 static void
-cb_sheet_object_raise (GtkWidget *widget, GObject *so_view)
+cb_so_pull_to_front (SheetObject *so, SheetControl *sc)
 {
-	SheetObject *so;
-	WorkbookControl *wbc;
-
-	so = sheet_object_view_obj (so_view);
-	wbc = sc_wbc (sheet_object_view_control (so_view));
-
-	cmd_object_raise (wbc, so, cmd_object_raise_up);
+	cmd_object_raise (sc_wbc (sc), so, cmd_object_pull_to_front);
 }
-
 static void
-cb_sheet_object_raise_to_top (GtkWidget *widget, GObject *so_view)
+cb_so_pull_forward (SheetObject *so, SheetControl *sc)
 {
-	SheetObject *so;
-	WorkbookControl *wbc;
-
-	so = sheet_object_view_obj (so_view);
-	wbc = sc_wbc (sheet_object_view_control (so_view));
-
-	cmd_object_raise (wbc, so, cmd_object_raise_top);
+	cmd_object_raise (sc_wbc (sc), so, cmd_object_pull_forward);
 }
-
 static void
-cb_sheet_object_lower (GtkWidget *widget, GObject *so_view)
+cb_so_push_backward (SheetObject *so, SheetControl *sc)
 {
-	SheetObject *so;
-	WorkbookControl *wbc;
-
-	so = sheet_object_view_obj (so_view);
-	wbc = sc_wbc (sheet_object_view_control (so_view));
-
-	cmd_object_raise (wbc, so, cmd_object_raise_down);
+	cmd_object_raise (sc_wbc (sc), so, cmd_object_push_backward);
 }
-
 static void
-cb_sheet_object_lower_to_bottom (GtkWidget *widget, GObject *so_view)
+cb_so_push_to_back (SheetObject *so, SheetControl *sc)
 {
-	SheetObject *so;
-	WorkbookControl *wbc;
-
-	so = sheet_object_view_obj (so_view);
-	wbc = sc_wbc (sheet_object_view_control (so_view));
-
-	cmd_object_raise (wbc, so, cmd_object_raise_bottom);
+	cmd_object_raise (sc_wbc (sc), so, cmd_object_push_to_back);
 }
-
 static void
-cb_sheet_object_remove (GtkWidget *widget, GObject *so_view)
+cb_so_delete (SheetObject *so, SheetControl *sc)
 {
-	cmd_object_delete (sc_wbc (sheet_object_view_control (so_view)),
-			   sheet_object_view_obj (so_view));
+	cmd_objects_delete (sc_wbc (sc), 
+		gnm_slist_create (so, NULL), NULL);
 }
-
 static void
-cb_sheet_object_configure (GtkWidget *widget, GObject *obj_view)
+cb_so_configure (SheetObject *so, SheetControl *sc)
 {
-	SheetControl *sc;
-	SheetObject *so;
-
-	g_return_if_fail (obj_view != NULL);
-
-	so = sheet_object_view_obj (obj_view);
-	sc = sheet_object_view_control (obj_view);
-
 	SO_CLASS(so)->user_config (so, sc);
 }
-
-/**
- * sheet_object_populate_menu:
- * @so:  the sheet object
- * @menu: the menu to insert into
- *
- * Add standard items to the object's popup menu.
- */
 static void
-sheet_object_populate_menu (SheetObject *so,
-			    GObject *obj_view,
-			    GtkMenu *menu)
+cb_so_cut (SheetObject *so, SheetControl *sc)
 {
-	GtkWidget *item;
-	if (SO_CLASS(so)->user_config != NULL) {
-		item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PROPERTIES, NULL);
-		g_signal_connect (G_OBJECT (item),
-			"activate",
-			G_CALLBACK (cb_sheet_object_configure), obj_view);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu),  item);
-	}
-
-	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_GOTO_TOP, NULL);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu),  item);
-	g_signal_connect (G_OBJECT (item),
-			  "activate",
-			  G_CALLBACK (cb_sheet_object_raise_to_top), obj_view);
-	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_GO_UP, NULL);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu),  item);
-	g_signal_connect (G_OBJECT (item),
-			  "activate",
-			  G_CALLBACK (cb_sheet_object_raise), obj_view);
-	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_GO_DOWN, NULL);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu),  item);
-	g_signal_connect (G_OBJECT (item),
-			  "activate",
-			  G_CALLBACK (cb_sheet_object_lower), obj_view);
-	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_GOTO_BOTTOM, NULL);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu),  item);
-	g_signal_connect (G_OBJECT (item),
-			  "activate",
-			  G_CALLBACK (cb_sheet_object_lower_to_bottom), obj_view);
-	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_DELETE, NULL);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu),  item);
-	g_signal_connect (G_OBJECT (item),
-		"activate",
-		G_CALLBACK (cb_sheet_object_remove), obj_view);
+	gnm_app_clipboard_cut_copy_obj (sc_wbc (sc), TRUE, sc_view (sc),
+		gnm_slist_create (so, NULL));
+}
+static void
+cb_so_copy (SheetObject *so, SheetControl *sc)
+{
+	gnm_app_clipboard_cut_copy_obj (sc_wbc (sc), FALSE, sc_view (sc),
+		gnm_slist_create (so, NULL));
 }
 
-/**
- * sheet_object_unrealize:
- *
- * Clears all views of this object in its current sheet's controls.
- */
 static void
-sheet_object_unrealize (SheetObject *so)
+sheet_object_populate_menu (SheetObject *so, GPtrArray *actions)
 {
-	g_return_if_fail (IS_SHEET_OBJECT (so));
-
-	/* The views remove themselves from the list */
-	while (so->realized_list != NULL)
-		sc_object_destroy_view (
-			sheet_object_view_control (G_OBJECT (so->realized_list->data)),
-			so);
+	static SheetObjectAction const so_actions [] = {
+		{ "gtk-properties",	NULL,		NULL,  0, cb_so_configure },
+		{ NULL,	NULL, NULL, 0, NULL },
+		{ NULL,			N_("_Order"),	NULL,  1, NULL },
+			{ NULL,			N_("Pul_l to Front"),	NULL,  0, cb_so_pull_to_front },
+			{ NULL,			N_("Pull _Forward"),	NULL,  0, cb_so_pull_forward },
+			{ NULL,			N_("Push _Backward"),	NULL,  0, cb_so_push_backward },
+			{ NULL,			N_("Pus_h to Back"),	NULL,  0, cb_so_push_to_back },
+			{ NULL,			NULL,			NULL, -1, NULL },
+		{ NULL,	NULL, NULL, 0, NULL },
+		{ "gtk-cut",		NULL,		NULL,  0, cb_so_cut },
+		{ "gtk-copy",		NULL,		NULL,  0, cb_so_copy },
+		{ "gtk-delete",		NULL,		NULL, 0, cb_so_delete },
+	};
+	unsigned i;
+	for (i = 0 ; i < G_N_ELEMENTS (so_actions); i++)
+		if (i != 0 || SO_CLASS(so)->user_config != NULL)
+			g_ptr_array_add (actions, (gpointer) (so_actions + i));
 }
 
 /**
@@ -220,27 +153,8 @@ static void
 sheet_object_finalize (GObject *object)
 {
 	SheetObject *so = SHEET_OBJECT (object);
-
-	g_return_if_fail (so != NULL);
-
-	sheet_object_unrealize (so);
-
-	if (so->sheet != NULL) {
-		g_return_if_fail (IS_SHEET (so->sheet));
-
-		/* If the object has already been inserted then mark sheet as dirty */
-		if (NULL != g_list_find	(so->sheet->sheet_objects, so)) {
-			so->sheet->sheet_objects =
-				g_list_remove (so->sheet->sheet_objects, so);
-			so->sheet->modified = TRUE;
-		}
-
-		if (so->anchor.cell_bound.end.col == so->sheet->max_object_extent.col &&
-		    so->anchor.cell_bound.end.row == so->sheet->max_object_extent.row)
-			sheet_objects_max_extent (so->sheet);
-		so->sheet = NULL;
-	}
-
+	if (so->sheet != NULL)
+		sheet_object_clear_sheet (so);
 	(*parent_klass->finalize) (object);
 }
 
@@ -250,10 +164,9 @@ sheet_object_init (GObject *object)
 	int i;
 	SheetObject *so = SHEET_OBJECT (object);
 
-	so->type = SHEET_OBJECT_ACTION_STATIC;
 	so->sheet = NULL;
-	so->is_visible = TRUE;
-	so->move_with_cells = TRUE;
+	so->flags = SHEET_OBJECT_IS_VISIBLE | SHEET_OBJECT_PRINT |
+		SHEET_OBJECT_MOVE_WITH_CELLS | SHEET_OBJECT_SIZE_WITH_CELLS;
 
 	/* Store the logical position as A1 */
 	so->anchor.cell_bound.start.col = so->anchor.cell_bound.start.row = 0;
@@ -295,43 +208,30 @@ sheet_object_class_init (GObjectClass *klass)
 		(GSignalAccumulator) NULL, NULL,
 		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE, 0);
+	signals [UNREALIZED] = g_signal_new ("unrealized",
+		SHEET_OBJECT_TYPE,
+		G_SIGNAL_RUN_LAST,
+		G_STRUCT_OFFSET (SheetObjectClass, unrealized),
+		(GSignalAccumulator) NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE, 0);
 }
 
 GSF_CLASS (SheetObject, sheet_object,
 	   sheet_object_class_init, sheet_object_init,
 	   G_TYPE_OBJECT);
 
-SheetObject *
-sheet_object_view_obj (GObject *view)
-{
-	gpointer obj = g_object_get_data (view, SO_VIEW_OBJECT_KEY);
-	return SHEET_OBJECT (obj);
-}
-
-SheetControl *
-sheet_object_view_control (GObject *view)
-{
-	gpointer obj = g_object_get_data (view, SO_VIEW_SHEET_CONTROL_KEY);
-	return SHEET_CONTROL (obj);
-}
-
-gpointer
-sheet_object_view_key (GObject *view)
-{
-	return g_object_get_data (view, SO_VIEW_KEY);
-}
-
-GObject *
-sheet_object_get_view (SheetObject *so, gpointer key)
+SheetObjectView *
+sheet_object_get_view (SheetObject const *so, SheetObjectViewContainer *container)
 {
 	GList *l;
 
 	g_return_val_if_fail (IS_SHEET_OBJECT (so), NULL);
 
 	for (l = so->realized_list; l; l = l->next) {
-		GObject *obj = G_OBJECT (l->data);
-		if (key == g_object_get_data (obj, SO_VIEW_KEY))
-			return obj;
+		SheetObjectView *view = SHEET_OBJECT_VIEW (l->data);
+		if (container == g_object_get_qdata (G_OBJECT (view), sov_container_quark))
+			return view;
 	}
 
 	return NULL;
@@ -374,29 +274,42 @@ sheet_object_update_bounds (SheetObject *so, GnmCellPos const *pos)
 		i = so->anchor.cell_bound.start.row;
 		while (i <= end && is_hidden)
 			is_hidden &= sheet_row_is_hidden (so->sheet, i++);
-	}
-
-	so->is_visible = !is_hidden;
+		so->flags |= SHEET_OBJECT_IS_VISIBLE;
+	} else
+		so->flags &= ~SHEET_OBJECT_IS_VISIBLE;
 
 	g_signal_emit (so, signals [BOUNDS_CHANGED], 0);
 }
 
+/**
+ * sheet_object_get_sheet :
+ *
+ * A small utility to help keep the implementation of SheetObjects modular.
+ **/
+Sheet *
+sheet_object_get_sheet (SheetObject const *so)
+{
+	g_return_val_if_fail (IS_SHEET_OBJECT (so), NULL);
+
+	return so->sheet;
+}
+
+
 /**
  * sheet_object_set_sheet :
  * @so :
  * @sheet :
  *
  * Adds a reference to the object.
- */
+ *
+ * Returns TRUE if there was a problem
+ **/
 gboolean
 sheet_object_set_sheet (SheetObject *so, Sheet *sheet)
 {
 	g_return_val_if_fail (IS_SHEET_OBJECT (so), TRUE);
 	g_return_val_if_fail (IS_SHEET (sheet), TRUE);
-
-	if (so->sheet == sheet)
-		return TRUE;
-
+	g_return_val_if_fail (so->sheet == NULL, TRUE);
 	g_return_val_if_fail (g_list_find (sheet->sheet_objects, so) == NULL, TRUE);
 
 	so->sheet = sheet;
@@ -418,44 +331,43 @@ sheet_object_set_sheet (SheetObject *so, Sheet *sheet)
 	return FALSE;
 }
 
-/**
- * sheet_object_get_sheet :
- *
- * A small utility to help keep the implementation of SheetObjects modular.
- */
-Sheet *
-sheet_object_get_sheet (SheetObject const *so)
-{
-	g_return_val_if_fail (IS_SHEET_OBJECT (so), NULL);
-
-	return so->sheet;
-}
-
 /**
  * sheet_object_clear_sheet :
  * @so :
- */
+ *
+ * Removes @so from it's container, unrealizes all views, disconects the
+ * associated data and unrefs the object
+ *
+ * Returns TRUE if there was a problem
+ **/
 gboolean
 sheet_object_clear_sheet (SheetObject *so)
 {
 	GList *ptr;
 
 	g_return_val_if_fail (IS_SHEET_OBJECT (so), TRUE);
-
-	if (!IS_SHEET (so->sheet))
-		return FALSE;
+	g_return_val_if_fail (IS_SHEET (so->sheet), TRUE);
 
 	ptr = g_list_find (so->sheet->sheet_objects, so);
 	g_return_val_if_fail (ptr != NULL, TRUE);
 
+	/* The views remove themselves from the list */
+	while (so->realized_list != NULL)
+		sheet_object_view_destroy (so->realized_list->data);
+	g_signal_emit (so, signals [UNREALIZED], 0);
+
 	if (SO_CLASS (so)->remove_from_sheet &&
 	    SO_CLASS (so)->remove_from_sheet (so))
 		return TRUE;
 
-	sheet_object_unrealize (so);
 	so->sheet->sheet_objects = g_list_remove_link (so->sheet->sheet_objects, ptr);
-	so->sheet = NULL;
 	g_list_free (ptr);
+
+	if (so->anchor.cell_bound.end.col == so->sheet->max_object_extent.col &&
+	    so->anchor.cell_bound.end.row == so->sheet->max_object_extent.row)
+		sheet_objects_max_extent (so->sheet);
+
+	so->sheet = NULL;
 	g_object_unref (G_OBJECT (so));
 
 	return FALSE;
@@ -467,35 +379,46 @@ cb_sheet_object_view_finalized (SheetObject *so, GObject *view)
 	so->realized_list = g_list_remove (so->realized_list, view);
 }
 
-/*
- * sheet_object_new_view
+/**
+ * sheet_object_new_view:
+ * @so :
+ * @sc :
+ * @key :
  *
- * Creates a FooCanvasItem for a SheetControlGUI and sets up the event
- * handlers.
- */
+ * Asks @so to create a view for the (@sc,@key) pair.
+ **/
 void
-sheet_object_new_view (SheetObject *so, SheetControl *sc, gpointer key)
+sheet_object_new_view (SheetObject *so, SheetObjectViewContainer *container)
 {
-	GObject *view;
+	SheetObjectView *view;
 
-	g_return_if_fail (IS_SHEET_CONTROL (sc));
 	g_return_if_fail (IS_SHEET_OBJECT (so));
+	g_return_if_fail (NULL != container);
 
-	view = sheet_object_get_view (so, key);
+	view = sheet_object_get_view (so, container);
 	if (view != NULL)
 		return;
 
-	view = SO_CLASS (so)->new_view (so, sc, key);
+	view = SO_CLASS (so)->new_view (so, container);
 
-	g_return_if_fail (GTK_IS_OBJECT (view));
+	g_return_if_fail (IS_SHEET_OBJECT_VIEW (view));
 
 	/* Store some useful information */
-	g_object_set_data (G_OBJECT (view), SO_VIEW_OBJECT_KEY, so);
-	g_object_set_data (G_OBJECT (view), SO_VIEW_SHEET_CONTROL_KEY, sc);
-	g_object_set_data (G_OBJECT (view), SO_VIEW_KEY, key);
+	g_object_set_qdata (G_OBJECT (view), sov_so_quark, so);
+	g_object_set_qdata (G_OBJECT (view), sov_container_quark, container);
 	g_object_weak_ref (G_OBJECT (view),
 		(GWeakNotify) cb_sheet_object_view_finalized, so);
 	so->realized_list = g_list_prepend (so->realized_list, view);
+	sheet_object_update_bounds (so, NULL);
+}
+
+gboolean
+sheet_object_can_print (SheetObject const *so)
+{
+	g_return_val_if_fail (IS_SHEET_OBJECT (so), FALSE);
+	return  (so->flags & SHEET_OBJECT_IS_VISIBLE) &&
+		(so->flags & SHEET_OBJECT_PRINT) &&
+		SO_CLASS (so)->print != NULL;
 }
 
 void
@@ -507,7 +430,7 @@ sheet_object_print (SheetObject const *so, GnomePrintContext *ctx,
 }
 
 GnmRange const *
-sheet_object_range_get (SheetObject const *so)
+sheet_object_get_range (SheetObject const *so)
 {
 	g_return_val_if_fail (IS_SHEET_OBJECT (so), NULL);
 
@@ -515,7 +438,7 @@ sheet_object_range_get (SheetObject const *so)
 }
 
 SheetObjectAnchor const *
-sheet_object_anchor_get (SheetObject const *so)
+sheet_object_get_anchor (SheetObject const *so)
 {
 	g_return_val_if_fail (IS_SHEET_OBJECT (so), NULL);
 
@@ -523,7 +446,7 @@ sheet_object_anchor_get (SheetObject const *so)
 }
 
 void
-sheet_object_anchor_set (SheetObject *so, SheetObjectAnchor const *anchor)
+sheet_object_set_anchor (SheetObject *so, SheetObjectAnchor const *anchor)
 {
 	g_return_if_fail (IS_SHEET_OBJECT (so));
 
@@ -543,54 +466,6 @@ sheet_object_anchor_cpy	(SheetObjectAnchor *dst, SheetObjectAnchor const *src)
 	memcpy (dst, src, sizeof (SheetObjectAnchor));
 }
 
-static int
-cell_offset_calc_pixel (Sheet const *sheet, int i, gboolean is_col,
-			SheetObjectAnchorType anchor_type, float offset)
-{
-	ColRowInfo const *cri = sheet_colrow_get_info (sheet, i, is_col);
-	/* TODO : handle other anchor types */
-	if (anchor_type == SO_ANCHOR_PERCENTAGE_FROM_COLROW_END)
-		return .5 + (1. - offset) * cri->size_pixels;
-	return offset * cri->size_pixels;
-}
-
-/**
- * sheet_object_position_pixels_get :
- *
- * @so : The sheet object
- * @coords : array of 4 ints where we return the coordinates in pixels
- *
- * Calculate the position of the object @so in pixels from the logical position
- * in the object.
- */
-void
-sheet_object_position_pixels_get (SheetObject const *so,
-				  SheetControl const *sc, double *coords)
-{
-	GnmRange const *r;
-
-	g_return_if_fail (IS_SHEET_OBJECT (so));
-	g_return_if_fail (IS_SHEET (so->sheet));
-
-	r = &so->anchor.cell_bound;
-
-	coords [0] = sc_colrow_distance_get (sc, TRUE, 0,  r->start.col);
-	coords [2] = coords [0] + sc_colrow_distance_get (sc, TRUE,
-		r->start.col, r->end.col);
-	coords [1] = sc_colrow_distance_get (sc, FALSE, 0, r->start.row);
-	coords [3] = coords [1] + sc_colrow_distance_get (sc, FALSE,
-		r->start.row, r->end.row);
-
-	coords [0] += cell_offset_calc_pixel (so->sheet, r->start.col,
-		TRUE, so->anchor.type [0], so->anchor.offset [0]);
-	coords [1] += cell_offset_calc_pixel (so->sheet, r->start.row,
-		FALSE, so->anchor.type [1], so->anchor.offset [1]);
-	coords [2] += cell_offset_calc_pixel (so->sheet, r->end.col,
-		TRUE, so->anchor.type [2], so->anchor.offset [2]);
-	coords [3] += cell_offset_calc_pixel (so->sheet, r->end.row,
-		FALSE, so->anchor.type [3], so->anchor.offset [3]);
-}
-
 static double
 cell_offset_calc_pt (Sheet const *sheet, int i, gboolean is_col,
 		     SheetObjectAnchorType anchor_type, float offset)
@@ -609,7 +484,7 @@ cell_offset_calc_pt (Sheet const *sheet, int i, gboolean is_col,
  * @h : a ptr into which to store the default_height.
  *
  * Measurements are in pts.
- */
+ **/
 void
 sheet_object_default_size (SheetObject *so, double *w, double *h)
 {
@@ -628,34 +503,41 @@ sheet_object_default_size (SheetObject *so, double *w, double *h)
  *
  * Calculate the position of the object @so in pts from the logical position in
  * the object.
- */
+ **/
 void
 sheet_object_position_pts_get (SheetObject const *so, double *coords)
+{
+	g_return_if_fail (IS_SHEET_OBJECT (so));
+	sheet_object_anchor_to_pts (&so->anchor, so->sheet, coords);
+}
+
+void
+sheet_object_anchor_to_pts (SheetObjectAnchor const *anchor,
+			    Sheet const *sheet, double *res_pts)
 {
 	GnmRange const *r;
 
-	g_return_if_fail (IS_SHEET_OBJECT (so));
-	g_return_if_fail (coords != NULL);
+	g_return_if_fail (res_pts != NULL);
 
-	r = &so->anchor.cell_bound;
+	r = &anchor->cell_bound;
 
-	coords [0] = sheet_col_get_distance_pts (so->sheet, 0,
+	res_pts [0] = sheet_col_get_distance_pts (sheet, 0,
 		r->start.col);
-	coords [2] = coords [0] + sheet_col_get_distance_pts (so->sheet,
+	res_pts [2] = res_pts [0] + sheet_col_get_distance_pts (sheet,
 		r->start.col, r->end.col);
-	coords [1] = sheet_row_get_distance_pts (so->sheet, 0,
+	res_pts [1] = sheet_row_get_distance_pts (sheet, 0,
 		r->start.row);
-	coords [3] = coords [1] + sheet_row_get_distance_pts (so->sheet,
+	res_pts [3] = res_pts [1] + sheet_row_get_distance_pts (sheet,
 		r->start.row, r->end.row);
 
-	coords [0] += cell_offset_calc_pt (so->sheet, r->start.col,
-		TRUE, so->anchor.type [0], so->anchor.offset [0]);
-	coords [1] += cell_offset_calc_pt (so->sheet, r->start.row,
-		FALSE, so->anchor.type [1], so->anchor.offset [1]);
-	coords [2] += cell_offset_calc_pt (so->sheet, r->end.col,
-		TRUE, so->anchor.type [2], so->anchor.offset [2]);
-	coords [3] += cell_offset_calc_pt (so->sheet, r->end.row,
-		FALSE, so->anchor.type [3], so->anchor.offset [3]);
+	res_pts [0] += cell_offset_calc_pt (sheet, r->start.col,
+		TRUE, anchor->type [0], anchor->offset [0]);
+	res_pts [1] += cell_offset_calc_pt (sheet, r->start.row,
+		FALSE, anchor->type [1], anchor->offset [1]);
+	res_pts [2] += cell_offset_calc_pt (sheet, r->end.col,
+		TRUE, anchor->type [2], anchor->offset [2]);
+	res_pts [3] += cell_offset_calc_pt (sheet, r->end.row,
+		FALSE, anchor->type [3], anchor->offset [3]);
 }
 
 /**
@@ -664,12 +546,14 @@ sheet_object_position_pts_get (SheetObject const *so, double *coords)
  * @rinfo : details on what should be moved.
  * @update : Should we do the bound_update now, or leave it for later.
  * 		if FALSE honour the move_with_cells flag.
+ * @undo : if non-NULL add dropped objects to ::objects
  *
  * Uses the relocation info and the anchors to decide whether or not, and how
  * to relocate objects when the grid moves (eg ins/del col/row).
- */
+ **/
 void
-sheet_objects_relocate (GnmExprRelocateInfo const *rinfo, gboolean update)
+sheet_objects_relocate (GnmExprRelocateInfo const *rinfo, gboolean update,
+			GnmRelocUndo *undo)
 {
 	GList   *ptr, *next;
 	GnmRange	 dest;
@@ -689,8 +573,11 @@ sheet_objects_relocate (GnmExprRelocateInfo const *rinfo, gboolean update)
 		for (ptr = copy; ptr != NULL ; ptr = ptr->next ) {
 			SheetObject *so = SHEET_OBJECT (ptr->data);
 			GnmRange const *r  = &so->anchor.cell_bound;
-			if (range_contains (&dest, r->start.col, r->start.row))
-				g_object_unref (G_OBJECT (so));
+			if (range_contains (&dest, r->start.col, r->start.row)) {
+				if (NULL != undo)
+					undo->objs = g_slist_prepend (undo->objs, g_object_ref (so));
+				sheet_object_clear_sheet (so);
+			}
 		}
 		g_list_free (copy);
 	}
@@ -701,24 +588,30 @@ sheet_objects_relocate (GnmExprRelocateInfo const *rinfo, gboolean update)
 		GnmRange       *r  = &so->anchor.cell_bound;
 
 		next = ptr->next;
-		if (update && !so->move_with_cells)
+		if (update && 0 == (so->flags & SHEET_OBJECT_MOVE_WITH_CELLS))
 			continue;
 		if (range_contains (&rinfo->origin,
 				    r->start.col, r->start.row)) {
 			/* FIXME : just moving the range is insufficent for all anchor types */
 			/* Toss any objects that would be clipped. */
 			if (range_translate (r, rinfo->col_offset, rinfo->row_offset)) {
-				g_object_unref (G_OBJECT (so));
+				if (NULL != undo)
+					undo->objs = g_slist_prepend (undo->objs, g_object_ref (so));
+				sheet_object_clear_sheet (so);
 				continue;
 			}
 			if (change_sheets) {
+				g_object_ref (so);
 				sheet_object_clear_sheet (so);
 				sheet_object_set_sheet (so, rinfo->target_sheet);
+				g_object_unref (so);
 			} else if (update)
 				sheet_object_update_bounds (so, NULL);
 		} else if (!change_sheets &&
 			   range_contains (&dest, r->start.col, r->start.row)) {
-			g_object_unref (G_OBJECT (so));
+			if (NULL != undo)
+				undo->objs = g_slist_prepend (undo->objs, g_object_ref (so));
+			sheet_object_clear_sheet (so);
 			continue;
 		}
 	}
@@ -736,8 +629,9 @@ sheet_objects_relocate (GnmExprRelocateInfo const *rinfo, gboolean update)
  * @t     : The type of object to lookup
  *
  * Returns a list of which the caller must free (just the list not the content).
- * Containing all objects of exactly the specified type (inheritence does not count).
- */
+ * Containing all objects of exactly the specified type (inheritence does not count)
+ * that are completely contained by @r.
+ **/
 GSList *
 sheet_objects_get (Sheet const *sheet, GnmRange const *r, GType t)
 {
@@ -751,7 +645,7 @@ sheet_objects_get (Sheet const *sheet, GnmRange const *r, GType t)
 
 		if (t == G_TYPE_NONE || t == G_OBJECT_TYPE (obj)) {
 			SheetObject *so = SHEET_OBJECT (obj);
-			if (r == NULL || range_overlap (r, &so->anchor.cell_bound))
+			if (r == NULL || range_contained (&so->anchor.cell_bound, r))
 				res = g_slist_prepend (res, so);
 		}
 	}
@@ -759,13 +653,13 @@ sheet_objects_get (Sheet const *sheet, GnmRange const *r, GType t)
 }
 
 /**
- * sheet_object_clear :
+ * sheet_objects_clear :
  *
  * @sheet : the sheet.
  * @r     : an optional range to look in
  *
  * removes the objects in the region.
- */
+ **/
 void
 sheet_objects_clear (Sheet const *sheet, GnmRange const *r, GType t)
 {
@@ -778,50 +672,36 @@ sheet_objects_clear (Sheet const *sheet, GnmRange const *r, GType t)
 		next = ptr->next;
 		if (t == G_TYPE_NONE || t == G_OBJECT_TYPE (obj)) {
 			SheetObject *so = SHEET_OBJECT (obj);
-			if (r == NULL || range_overlap (r, &so->anchor.cell_bound))
-				g_object_unref (G_OBJECT (so));
+			if (r == NULL || range_contained (&so->anchor.cell_bound, r)) {
+				/* lost_objs = g_slist_prepend (lost_objs, g_object_ref (so)); */
+				sheet_object_clear_sheet (so);
+			}
 		}
 	}
 }
 
-void
-sheet_object_register (void)
-{
-	SHEET_OBJECT_GRAPHIC_TYPE;
-	SHEET_OBJECT_FILLED_TYPE;
-	SHEET_OBJECT_TEXT_TYPE;
-	SHEET_OBJECT_GRAPH_TYPE;
-	SHEET_OBJECT_IMAGE_TYPE;
-	GNM_GO_DATA_SCALAR_TYPE;
-	GNM_GO_DATA_VECTOR_TYPE;
-	CELL_COMMENT_TYPE;
-
-	sheet_object_widget_register ();
-}
-
 /**
- * sheet_object_clone:
- * @so: The Sheet Object to clone
- * @sheet: The sheet that we should attach the sheet object to
+ * sheet_object_dup:
+ * @so: a #SheetObject to duplicate
  *
- * Clones a sheet object and attaches it to @sheet
- *
- * Return Value:
+ * Returns : A copy of @so that is not attached to a sheet.
+ *    Caller is responsible for the reference.
  **/
-static SheetObject *
-sheet_object_clone (SheetObject const *so, Sheet *sheet)
+SheetObject *
+sheet_object_dup (SheetObject const *so)
 {
 	SheetObject *new_so = NULL;
 
-	if (!SO_CLASS (so)->clone)
+	if (!SO_CLASS (so)->copy)
 		return NULL;
 
-	new_so = SO_CLASS (so)->clone (so, sheet);
+	new_so = g_object_new (G_OBJECT_TYPE (so), NULL);
 
-	new_so->type = so->type;
+	g_return_val_if_fail (new_so != NULL, NULL);
+
+	SO_CLASS (so)->copy (new_so, so);
+	new_so->flags = so->flags;
 	sheet_object_anchor_cpy (&new_so->anchor, &so->anchor);
-	sheet_object_set_sheet (new_so, sheet);
-	g_object_unref (new_so);
 
 	return new_so;
 }
@@ -841,7 +721,6 @@ sheet_object_clone_sheet (Sheet const *src, Sheet *dst, GnmRange *range)
 	SheetObject *so;
 	SheetObject *new_so;
 	GList *list;
-	GList *new_list = NULL;
 
 	g_return_if_fail (IS_SHEET (dst));
 	g_return_if_fail (dst->sheet_objects == NULL);
@@ -850,13 +729,15 @@ sheet_object_clone_sheet (Sheet const *src, Sheet *dst, GnmRange *range)
 	for (; list != NULL; list = list->next) {
 		so = (SheetObject *) list->data;
 		if (range == NULL || range_overlap (range, &so->anchor.cell_bound)) {
-			new_so = sheet_object_clone (so, dst);
-			if (new_so != NULL)
-				new_list = g_list_prepend (new_list, new_so);
+			new_so = sheet_object_dup (so);
+			if (new_so != NULL) {
+				sheet_object_set_sheet (new_so, dst);
+				g_object_unref (new_so);
+			}
 		}
 	}
 
-	dst->sheet_objects = g_list_reverse (new_list);
+	dst->sheet_objects = g_list_reverse (dst->sheet_objects);
 }
 
 
@@ -868,10 +749,9 @@ sheet_object_clone_sheet (Sheet const *src, Sheet *dst, GnmRange *range)
  * Sets the object direction from the given the new coordinates
  * The original coordinates are assumed to be normalized (so that top
  * is above bottom and right is at the right of left)
- *
  **/
 void
-sheet_object_direction_set (SheetObject *so, gdouble *coords)
+sheet_object_direction_set (SheetObject *so, gdouble const *coords)
 {
 	if (so->anchor.direction == SO_DIR_UNKNOWN)
 		return;
@@ -929,10 +809,10 @@ sheet_object_anchor_init (SheetObjectAnchor *anchor,
 
 	if (types == NULL) {
 		static SheetObjectAnchorType const defaultVal [4] = {
-			SO_ANCHOR_PTS_FROM_COLROW_START,
-			SO_ANCHOR_PTS_FROM_COLROW_START,
-			SO_ANCHOR_PTS_FROM_COLROW_START,
-			SO_ANCHOR_PTS_FROM_COLROW_START
+			SO_ANCHOR_PERCENTAGE_FROM_COLROW_START,
+			SO_ANCHOR_PERCENTAGE_FROM_COLROW_START,
+			SO_ANCHOR_PERCENTAGE_FROM_COLROW_START,
+			SO_ANCHOR_PERCENTAGE_FROM_COLROW_START
 		};
 		types = defaultVal;
 	}
@@ -945,6 +825,22 @@ sheet_object_anchor_init (SheetObjectAnchor *anchor,
 
 /*****************************************************************************/
 
+gint
+sheet_object_get_stacking (SheetObject *so)
+{
+	GList *l = so->realized_list;
+
+	if (l != NULL) {
+		FooCanvasItem *item = FOO_CANVAS_ITEM (l->data);
+		FooCanvasGroup *parent = FOO_CANVAS_GROUP (item->parent);
+		GList *link = g_list_find (parent->item_list, item);
+		return g_list_position (parent->item_list, link);
+	}
+	return -1;
+}
+
+/*****************************************************************************/
+
 gint
 sheet_object_adjust_stacking (SheetObject *so, gint positions)
 {
@@ -966,3 +862,68 @@ sheet_object_adjust_stacking (SheetObject *so, gint positions)
 	}
 	return ((before == -1 || after == -1) ? positions : (after - before));
 }
+
+/*****************************************************************************/
+
+void
+sheet_object_view_set_bounds (SheetObjectView *sov,
+			      double const *coords, gboolean visible)
+{
+	SheetObjectViewIface *iface = SHEET_OBJECT_VIEW_GET_CLASS (sov);
+	iface->set_bounds (sov, coords, visible);
+}
+
+SheetObject *
+sheet_object_view_get_so (SheetObjectView *view)
+{
+	return g_object_get_qdata (G_OBJECT (view), sov_so_quark);
+}
+
+void
+sheet_object_view_destroy (SheetObjectView *sov)
+{
+	SheetObjectViewIface *iface = SHEET_OBJECT_VIEW_GET_CLASS (sov);
+	g_return_if_fail (iface != NULL);
+	if (iface->destroy)
+		iface->destroy (sov);
+}
+
+GType
+sheet_object_view_get_type (void)
+{
+	static GType type = 0;
+
+	if (!type) {
+		static GTypeInfo const type_info = {
+			sizeof (SheetObjectViewIface),	/* class_size */
+			NULL,				/* base_init */
+			NULL,				/* base_finalize */
+		};
+
+		type = g_type_register_static (G_TYPE_INTERFACE,
+			"SheetObjectView", &type_info, 0);
+	}
+
+	return type;
+}
+
+
+/*****************************************************************************/
+
+void
+sheet_objects_init (void)
+{
+	GNM_SO_LINE_TYPE;
+	GNM_SO_FILLED_TYPE;
+	SHEET_OBJECT_GRAPH_TYPE;
+	SHEET_OBJECT_IMAGE_TYPE;
+	GNM_GO_DATA_SCALAR_TYPE;
+	GNM_GO_DATA_VECTOR_TYPE;
+	GNM_GO_DATA_MATRIX_TYPE;
+	CELL_COMMENT_TYPE;
+
+	sheet_object_widget_register ();
+	sov_so_quark = g_quark_from_static_string ("SheetObject");
+	sov_container_quark = g_quark_from_static_string ("SheetObjectViewContainer");
+}
+
